# Copyright (C) 2009, 2010, 2013, 2014 Nicira Networks, Inc.
#
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.  This file is offered as-is,
# without warranty of any kind.
#
# If tests have to be skipped while building, specify the '--without check'
# option. For example:
# rpmbuild -bb --without check rhel/openvswitch-fedora.spec

# This defines the base package name's version.

%define pkgname openvswitch3.0


%if 0%{?commit:1}
%global shortcommit %(c=%{commit}; echo ${c:0:7})
%endif

# Enable PIE, bz#955181
%global _hardened_build 1

# RHEL-7 doesn't define _rundir macro yet
# Fedora 15 onwards uses /run as _rundir
%if 0%{!?_rundir:1}
%define _rundir /run
%endif

# FIXME Test "STP - flush the fdb and mdb when topology changed" fails on s390x
# FIXME 2 tests fails on ppc64le. They will be hopefully fixed before official 2.11
%ifarch %{ix86} x86_64 aarch64
%bcond_without check
%else
%bcond_with check
%endif
# option to run kernel datapath tests, requires building as root!
%bcond_with check_datapath_kernel
# option to build with libcap-ng, needed for running OVS as regular user
%bcond_without libcapng
# option to build with ipsec support
%bcond_without ipsec

# Build python2 (that provides python) and python3 subpackages on Fedora
# Build only python3 (that provides python) subpackage on RHEL8
# Build only python subpackage on RHEL7
%if 0%{?rhel} > 7 || 0%{?fedora}
# On RHEL8 Sphinx is included in buildroot
%global external_sphinx 1
%else
# Don't use external sphinx (RHV doesn't have optional repositories enabled)
%global external_sphinx 0
%endif

Name: %{pkgname}
Summary: Open vSwitch
Group: System Environment/Daemons daemon/database/utilities
URL: http://www.openvswitch.org/
Version: 3.0.0
Release: 27%{?dist}

# Nearly all of openvswitch is ASL 2.0.  The bugtool is LGPLv2+, and the
# lib/sflow*.[ch] files are SISSL
# datapath/ is GPLv2 (although not built into any of the binary packages)
License: ASL 2.0 and LGPLv2+ and SISSL

%define dpdkver 21.11.2
%define dpdkdir dpdk
%define dpdksver %(echo %{dpdkver} | cut -d. -f-2)
# NOTE: DPDK does not currently build for s390x
# DPDK on aarch64 is not stable enough to be enabled in FDP
%if 0%{?rhel} > 7 || 0%{?fedora}
%define dpdkarches x86_64 ppc64le
%else
%define dpdkarches
%endif

%if 0%{?commit:1}
Source: https://github.com/openvswitch/ovs/archive/%{commit}.tar.gz#/openvswitch-%{commit}.tar.gz
%else
Source: https://github.com/openvswitch/ovs/archive/v%{version}.tar.gz#/openvswitch-%{version}.tar.gz
%endif
Source10: https://fast.dpdk.org/rel/dpdk-%{dpdkver}.tar.xz

%define docutilsver 0.12
%define pygmentsver 1.4
%define sphinxver   1.2.3
%define pyelftoolsver 0.27
Source100: https://pypi.io/packages/source/d/docutils/docutils-%{docutilsver}.tar.gz
Source101: https://pypi.io/packages/source/P/Pygments/Pygments-%{pygmentsver}.tar.gz
Source102: https://pypi.io/packages/source/S/Sphinx/Sphinx-%{sphinxver}.tar.gz
Source103: https://pypi.io/packages/source/p/pyelftools/pyelftools-%{pyelftoolsver}.tar.gz

Patch:     openvswitch-%{version}.patch

# The DPDK is designed to optimize througput of network traffic using, among
# other techniques, carefully crafted assembly instructions.  As such it
# needs extensive work to port it to other architectures.
ExclusiveArch: x86_64 aarch64 ppc64le s390x

# Do not enable this otherwise YUM will break on any upgrade.
# Provides: openvswitch
Conflicts: openvswitch < 3.0
Conflicts: openvswitch-dpdk < 3.0
Conflicts: openvswitch2.10
Conflicts: openvswitch2.11
Conflicts: openvswitch2.12
Conflicts: openvswitch2.13
Conflicts: openvswitch2.14
Conflicts: openvswitch2.15
Conflicts: openvswitch2.16
Conflicts: openvswitch2.17

# FIXME Sphinx is used to generate some manpages, unfortunately, on RHEL, it's
# in the -optional repository and so we can't require it directly since RHV
# doesn't have the -optional repository enabled and so TPS fails
%if %{external_sphinx}
BuildRequires: python3-sphinx
%else
# Sphinx dependencies
BuildRequires: python-devel
BuildRequires: python-setuptools
#BuildRequires: python2-docutils
BuildRequires: python-jinja2
BuildRequires: python-nose
#BuildRequires: python2-pygments
# docutils dependencies
BuildRequires: python-imaging
# pygments dependencies
BuildRequires: python-nose
%endif

BuildRequires: gcc gcc-c++ make
BuildRequires: autoconf automake libtool
BuildRequires: systemd-units openssl openssl-devel
BuildRequires: python3-devel python3-setuptools
BuildRequires: desktop-file-utils
BuildRequires: groff-base graphviz
BuildRequires: unbound-devel
BuildRequires: systemtap-sdt-devel
# make check dependencies
BuildRequires: procps-ng
%if %{with check_datapath_kernel}
BuildRequires: nmap-ncat
# would be useful but not available in RHEL or EPEL
#BuildRequires: pyftpdlib
%endif

%if %{with libcapng}
BuildRequires: libcap-ng libcap-ng-devel
%endif

%ifarch %{dpdkarches}
BuildRequires: meson
%if 0%{?rhel} > 8 || 0%{?fedora}
BuildRequires: python3-pyelftools
%endif
# DPDK driver dependencies
BuildRequires: zlib-devel numactl-devel libarchive-devel
%ifarch x86_64
BuildRequires: rdma-core-devel >= 15 libmnl-devel
%endif

# Required by packaging policy for the bundled DPDK
Provides: bundled(dpdk) = %{dpdkver}
%endif

Requires: openssl iproute module-init-tools
#Upstream kernel commit 4f647e0a3c37b8d5086214128614a136064110c3
#Requires: kernel >= 3.15.0-0
Requires: openvswitch-selinux-extra-policy

Requires(pre): shadow-utils
Requires(post): /bin/sed
Requires(post): /usr/sbin/usermod
Requires(post): /usr/sbin/groupadd
Requires(post): systemd-units
Requires(preun): systemd-units
Requires(postun): systemd-units
Obsoletes: openvswitch-controller <= 0:2.1.0-1

%if 0%{?rhel}
# sortedcontainers are not packaged on RHEL yet, but ovs includes it
%global __requires_exclude ^python%{python3_version}dist\\(sortedcontainers\\)$
%endif

%description
Open vSwitch provides standard network bridging functions and
support for the OpenFlow protocol for remote per-flow control of
traffic.

%package -n python3-%{pkgname}
Summary: Open vSwitch python3 bindings
License: ASL 2.0
Requires: %{pkgname} = %{?epoch:%{epoch}:}%{version}-%{release}
Provides: python-%{pkgname} = %{?epoch:%{epoch}:}%{version}-%{release}

%description -n python3-%{pkgname}
Python bindings for the Open vSwitch database

%package test
Summary: Open vSwitch testing utilities
License: ASL 2.0
BuildArch: noarch
Requires: python3-%{pkgname} = %{?epoch:%{epoch}:}%{version}-%{release}
Requires: tcpdump

%description test
Utilities that are useful to diagnose performance and connectivity
issues in Open vSwitch setup.

%package devel
Summary: Open vSwitch OpenFlow development package (library, headers)
License: ASL 2.0
Requires: %{pkgname} = %{?epoch:%{epoch}:}%{version}-%{release}

%description devel
This provides shared library, libopenswitch.so and the openvswitch header
files needed to build an external application.

%if 0%{?rhel} == 8 || 0%{?fedora} > 28
%package -n network-scripts-%{name}
Summary: Open vSwitch legacy network service support
License: ASL 2.0
Requires: network-scripts
Supplements: (%{name} and network-scripts)

%description -n network-scripts-%{name}
This provides the ifup and ifdown scripts for use with the legacy network
service.
%endif

%if %{with ipsec}
%package ipsec
Summary: Open vSwitch IPsec tunneling support
License: ASL 2.0
Requires: python3-%{pkgname} = %{?epoch:%{epoch}:}%{version}-%{release}
Requires: libreswan

%description ipsec
This package provides IPsec tunneling support for OVS tunnels.
%endif

%prep
%if 0%{?commit:1}
%setup -q -n ovs-%{commit} -a 10
%else
%setup -q -n ovs-%{version} -a 10
%endif
%if ! %{external_sphinx}
%if 0%{?commit:1}
%setup -n ovs-%{commit} -q -D -T -a 100 -a 101 -a 102
%else
%setup -n ovs-%{version} -q -D -T -a 100 -a 101 -a 102
%endif
%endif
%if 0%{?rhel} && 0%{?rhel} < 9
%if 0%{?commit:1}
%setup -n ovs-%{commit} -q -D -T -a 103
%else
%setup -n ovs-%{version} -q -D -T -a 103
%endif
%endif

mv dpdk-*/ %{dpdkdir}/

# FIXME should we propose a way to do that upstream?
sed -ri "/^subdir\('(usertools|app)'\)/d" %{dpdkdir}/meson.build

%patch -p1

%build
%if 0%{?rhel} && 0%{?rhel} < 9
export PYTHONPATH="${PWD}/pyelftools-%{pyelftoolsver}"
%endif
# Build Sphinx on RHEL
%if ! %{external_sphinx}
export PYTHONPATH="${PYTHONPATH:+$PYTHONPATH:}%{_builddir}/pytmp/lib/python"
for x in docutils-%{docutilsver} Pygments-%{pygmentsver} Sphinx-%{sphinxver}; do
    pushd "$x"
    python2 setup.py install --home %{_builddir}/pytmp
    popd
done

export PATH="$PATH:%{_builddir}/pytmp/bin"
%endif

./boot.sh

%ifarch %{dpdkarches}    # build dpdk
# Lets build DPDK first
cd %{dpdkdir}

ENABLED_DRIVERS=(
    bus/pci
    bus/vdev
    mempool/ring
    net/failsafe
    net/i40e
    net/ring
    net/vhost
    net/virtio
    net/tap
)

%ifarch x86_64
ENABLED_DRIVERS+=(
    bus/auxiliary
    bus/vmbus
    common/iavf
    common/mlx5
    net/bnxt
    net/enic
    net/iavf
    net/ice
    net/mlx5
    net/netvsc
    net/nfp
    net/qede
    net/vdev_netvsc
)
%endif

%ifarch aarch64 x86_64
ENABLED_DRIVERS+=(
    net/e1000
    net/ixgbe
)
%endif

for driver in "${ENABLED_DRIVERS[@]}"; do
    enable_drivers="${enable_drivers:+$enable_drivers,}"$driver
done

# As of 21.11-rc3, following libraries can be disabled:
# optional_libs = [
#         'bitratestats',
#         'gpudev',
#         'gro',
#         'gso',
#         'kni',
#         'jobstats',
#         'latencystats',
#         'metrics',
#         'pdump',
#         'power',
#         'vhost',
# ]
# If doing any updates, this must be aligned with:
# https://access.redhat.com/articles/3538141
DISABLED_LIBS=(
    gpudev
    kni
    jobstats
    power
)

for lib in "${DISABLED_LIBS[@]}"; do
    disable_libs="${disable_libs:+$disable_libs,}"$lib
done

%set_build_flags
%__meson --prefix=%{_builddir}/dpdk-build \
         --buildtype=plain \
         -Ddisable_libs="$disable_libs" \
         -Denable_drivers="$enable_drivers" \
         -Dplatform=generic \
         -Dmax_ethports=1024 \
         -Dmax_numa_nodes=8 \
         -Dtests=false \
         %{_vpath_builddir}
%meson_build
%__meson install -C %{_vpath_builddir} --no-rebuild

# FIXME currently with LTO enabled OVS tries to link with both static and shared libraries
rm -v %{_builddir}/dpdk-build/%{_lib}/*.so*

# Generate a list of supported drivers, its hard to tell otherwise.
cat << EOF > README.DPDK-PMDS
DPDK drivers included in this package:

EOF

for f in %{_builddir}/dpdk-build/%{_lib}/librte_net_*.a; do
    basename ${f} | cut -c12- | cut -d. -f1 | tr [:lower:] [:upper:]
done >> README.DPDK-PMDS

cat << EOF >> README.DPDK-PMDS

For further information about the drivers, see
http://dpdk.org/doc/guides-%{dpdksver}/nics/index.html
EOF

cd -
%endif    # build dpdk

# And now for OVS...
mkdir build-shared build-static
pushd build-shared
ln -s ../configure
%configure \
%if %{with libcapng}
        --enable-libcapng \
%else
        --disable-libcapng \
%endif
        --disable-static \
        --enable-shared \
        --enable-ssl \
        --with-pkidir=%{_sharedstatedir}/openvswitch/pki \
        --enable-usdt-probes
make %{?_smp_mflags}
popd
pushd build-static
ln -s ../configure
%ifarch %{dpdkarches}
PKG_CONFIG_PATH=%{_builddir}/dpdk-build/%{_lib}/pkgconfig \
%endif
%configure \
%if %{with libcapng}
        --enable-libcapng \
%else
        --disable-libcapng \
%endif
        --enable-ssl \
%ifarch %{dpdkarches}
        --with-dpdk=static \
%endif
        --with-pkidir=%{_sharedstatedir}/openvswitch/pki \
        --enable-usdt-probes
make %{?_smp_mflags}
popd

/usr/bin/python3 build-aux/dpdkstrip.py \
        --dpdk \
        < rhel/usr_lib_systemd_system_ovs-vswitchd.service.in \
        > rhel/usr_lib_systemd_system_ovs-vswitchd.service

%install
rm -rf $RPM_BUILD_ROOT
make -C build-shared install-libLTLIBRARIES DESTDIR=$RPM_BUILD_ROOT
make -C build-static install DESTDIR=$RPM_BUILD_ROOT

install -d -m 0755 $RPM_BUILD_ROOT%{_rundir}/openvswitch
install -d -m 0750 $RPM_BUILD_ROOT%{_localstatedir}/log/openvswitch
install -d -m 0755 $RPM_BUILD_ROOT%{_sysconfdir}/openvswitch

install -p -D -m 0644 rhel/usr_lib_udev_rules.d_91-vfio.rules \
        $RPM_BUILD_ROOT%{_udevrulesdir}/91-vfio.rules

install -p -D -m 0644 \
        rhel/usr_share_openvswitch_scripts_systemd_sysconfig.template \
        $RPM_BUILD_ROOT/%{_sysconfdir}/sysconfig/openvswitch

for service in openvswitch ovsdb-server ovs-vswitchd \
               ovs-delete-transient-ports; do
        install -p -D -m 0644 \
                        rhel/usr_lib_systemd_system_${service}.service \
                        $RPM_BUILD_ROOT%{_unitdir}/${service}.service
done

%if %{with ipsec}
install -p -D -m 0644 rhel/usr_lib_systemd_system_openvswitch-ipsec.service \
                      $RPM_BUILD_ROOT%{_unitdir}/openvswitch-ipsec.service
%endif

install -m 0755 rhel/etc_init.d_openvswitch \
        $RPM_BUILD_ROOT%{_datadir}/openvswitch/scripts/openvswitch.init

install -p -D -m 0644 rhel/etc_openvswitch_default.conf \
        $RPM_BUILD_ROOT/%{_sysconfdir}/openvswitch/default.conf

install -p -D -m 0644 rhel/etc_logrotate.d_openvswitch \
        $RPM_BUILD_ROOT/%{_sysconfdir}/logrotate.d/openvswitch

install -m 0644 vswitchd/vswitch.ovsschema \
        $RPM_BUILD_ROOT/%{_datadir}/openvswitch/vswitch.ovsschema

%if 0%{?rhel} < 9
install -d -m 0755 $RPM_BUILD_ROOT/%{_sysconfdir}/sysconfig/network-scripts/
install -p -m 0755 rhel/etc_sysconfig_network-scripts_ifdown-ovs \
        $RPM_BUILD_ROOT/%{_sysconfdir}/sysconfig/network-scripts/ifdown-ovs
install -p -m 0755 rhel/etc_sysconfig_network-scripts_ifup-ovs \
        $RPM_BUILD_ROOT/%{_sysconfdir}/sysconfig/network-scripts/ifup-ovs
%endif

install -d -m 0755 $RPM_BUILD_ROOT%{python3_sitelib}
cp -a $RPM_BUILD_ROOT/%{_datadir}/openvswitch/python/ovstest \
        $RPM_BUILD_ROOT%{python3_sitelib}

# Build the JSON C extension for the Python lib (#1417738)
pushd python
(
export CPPFLAGS="-I ../include -I ../build-shared/include"
export LDFLAGS="%{__global_ldflags} -L $RPM_BUILD_ROOT%{_libdir}"
%py3_build
%py3_install
[ -f "$RPM_BUILD_ROOT/%{python3_sitearch}/ovs/_json$(python3-config --extension-suffix)" ]
)
popd

rm -rf $RPM_BUILD_ROOT/%{_datadir}/openvswitch/python/

install -d -m 0755 $RPM_BUILD_ROOT/%{_sharedstatedir}/openvswitch

install -d -m 0755 $RPM_BUILD_ROOT%{_prefix}/lib/firewalld/services/

install -p -D -m 0755 \
        rhel/usr_share_openvswitch_scripts_ovs-systemd-reload \
        $RPM_BUILD_ROOT%{_datadir}/openvswitch/scripts/ovs-systemd-reload

touch $RPM_BUILD_ROOT%{_sysconfdir}/openvswitch/conf.db
# The db needs special permission as IPsec Pre-shared keys are stored in it.
chmod 0640 $RPM_BUILD_ROOT%{_sysconfdir}/openvswitch/conf.db

touch $RPM_BUILD_ROOT%{_sysconfdir}/openvswitch/system-id.conf

# remove unpackaged files
rm -f $RPM_BUILD_ROOT/%{_bindir}/ovs-benchmark \
        $RPM_BUILD_ROOT/%{_bindir}/ovs-docker \
        $RPM_BUILD_ROOT/%{_bindir}/ovs-parse-backtrace \
        $RPM_BUILD_ROOT/%{_bindir}/ovs-testcontroller \
        $RPM_BUILD_ROOT/%{_sbindir}/ovs-vlan-bug-workaround \
        $RPM_BUILD_ROOT/%{_mandir}/man1/ovs-benchmark.1* \
        $RPM_BUILD_ROOT/%{_mandir}/man8/ovs-testcontroller.* \
        $RPM_BUILD_ROOT/%{_mandir}/man8/ovs-vlan-bug-workaround.8*

%if ! %{with ipsec}
rm -f $RPM_BUILD_ROOT/%{_datadir}/openvswitch/scripts/ovs-monitor-ipsec
%endif

# remove ovn unpackages files
rm -f $RPM_BUILD_ROOT%{_bindir}/ovn*
rm -f $RPM_BUILD_ROOT%{_mandir}/man1/ovn*
rm -f $RPM_BUILD_ROOT%{_mandir}/man5/ovn*
rm -f $RPM_BUILD_ROOT%{_mandir}/man7/ovn*
rm -f $RPM_BUILD_ROOT%{_mandir}/man8/ovn*
rm -f $RPM_BUILD_ROOT%{_datadir}/openvswitch/ovn*
rm -f $RPM_BUILD_ROOT%{_datadir}/openvswitch/scripts/ovn*
rm -f $RPM_BUILD_ROOT%{_includedir}/ovn/*

%check
%if %{with check}
    pushd build-static
    touch resolv.conf
    export OVS_RESOLV_CONF=$(pwd)/resolv.conf
    if make check TESTSUITEFLAGS='%{_smp_mflags}' ||
       make check TESTSUITEFLAGS='--recheck'; then :;
    else
        cat tests/testsuite.log
        exit 1
    fi
    popd
%endif
%if %{with check_datapath_kernel}
    pushd build-static
    if make check-kernel RECHECK=yes; then :;
    else
        cat tests/system-kmod-testsuite.log
        exit 1
    fi
    popd
%endif

%clean
rm -rf $RPM_BUILD_ROOT

%preun
%if 0%{?systemd_preun:1}
    %systemd_preun openvswitch.service
%else
    if [ $1 -eq 0 ] ; then
    # Package removal, not upgrade
        /bin/systemctl --no-reload disable openvswitch.service >/dev/null 2>&1 || :
        /bin/systemctl stop openvswitch.service >/dev/null 2>&1 || :
    fi
%endif

%pre
getent group openvswitch >/dev/null || groupadd -r openvswitch
getent passwd openvswitch >/dev/null || \
    useradd -r -g openvswitch -d / -s /sbin/nologin \
    -c "Open vSwitch Daemons" openvswitch

%ifarch %{dpdkarches}
    getent group hugetlbfs >/dev/null || groupadd hugetlbfs
    usermod -a -G hugetlbfs openvswitch
%endif
exit 0

%post
if [ $1 -eq 1 ]; then
    sed -i 's:^#OVS_USER_ID=:OVS_USER_ID=:' /etc/sysconfig/openvswitch

%ifarch %{dpdkarches}
    sed -i \
        's@OVS_USER_ID="openvswitch:openvswitch"@OVS_USER_ID="openvswitch:hugetlbfs"@'\
        /etc/sysconfig/openvswitch
%endif
fi
chown -R openvswitch:openvswitch /etc/openvswitch

%if 0%{?systemd_post:1}
    %systemd_post openvswitch.service
%else
    # Package install, not upgrade
    if [ $1 -eq 1 ]; then
        /bin/systemctl daemon-reload >dev/null || :
    fi
%endif

%postun
%if 0%{?systemd_postun:1}
    %systemd_postun openvswitch.service
%else
    /bin/systemctl daemon-reload >/dev/null 2>&1 || :
%endif

%triggerun -- openvswitch < 2.5.0-22.git20160727%{?dist}
# old rpm versions restart the service in postun, but
# due to systemd some preparation is needed.
if systemctl is-active openvswitch >/dev/null 2>&1 ; then
    /usr/share/openvswitch/scripts/ovs-ctl stop >/dev/null 2>&1 || :
    systemctl daemon-reload >/dev/null 2>&1 || :
    systemctl stop openvswitch ovsdb-server ovs-vswitchd >/dev/null 2>&1 || :
    systemctl start openvswitch >/dev/null 2>&1 || :
fi
exit 0

%files -n python3-%{pkgname}
%{python3_sitearch}/ovs
%{python3_sitearch}/ovs-*.egg-info
%doc LICENSE

%files test
%{_bindir}/ovs-pcap
%{_bindir}/ovs-tcpdump
%{_bindir}/ovs-tcpundump
%{_mandir}/man1/ovs-pcap.1*
%{_mandir}/man8/ovs-tcpdump.8*
%{_mandir}/man1/ovs-tcpundump.1*
%{_bindir}/ovs-test
%{_bindir}/ovs-vlan-test
%{_bindir}/ovs-l3ping
%{_mandir}/man8/ovs-test.8*
%{_mandir}/man8/ovs-vlan-test.8*
%{_mandir}/man8/ovs-l3ping.8*
%{python3_sitelib}/ovstest

%files devel
%{_libdir}/*.so
%{_libdir}/pkgconfig/*.pc
%{_includedir}/openvswitch/*
%{_includedir}/openflow/*
%exclude %{_libdir}/*.a
%exclude %{_libdir}/*.la

%if 0%{?rhel} == 8 || 0%{?fedora} > 28
%files -n network-scripts-%{name}
%{_sysconfdir}/sysconfig/network-scripts/ifup-ovs
%{_sysconfdir}/sysconfig/network-scripts/ifdown-ovs
%endif

%files
%defattr(-,openvswitch,openvswitch)
%dir %{_sysconfdir}/openvswitch
%{_sysconfdir}/openvswitch/default.conf
%config %ghost %verify(not owner group md5 size mtime) %{_sysconfdir}/openvswitch/conf.db
%ghost %attr(0600,-,-) %verify(not owner group md5 size mtime) %{_sysconfdir}/openvswitch/.conf.db.~lock~
%config %ghost %{_sysconfdir}/openvswitch/system-id.conf
%defattr(-,root,root)
%config(noreplace) %verify(not md5 size mtime) %{_sysconfdir}/sysconfig/openvswitch
%{_sysconfdir}/bash_completion.d/ovs-appctl-bashcomp.bash
%{_sysconfdir}/bash_completion.d/ovs-vsctl-bashcomp.bash
%config(noreplace) %{_sysconfdir}/logrotate.d/openvswitch
%{_unitdir}/openvswitch.service
%{_unitdir}/ovsdb-server.service
%{_unitdir}/ovs-vswitchd.service
%{_unitdir}/ovs-delete-transient-ports.service
%{_datadir}/openvswitch/scripts/openvswitch.init
%{_datadir}/openvswitch/scripts/ovs-check-dead-ifs
%{_datadir}/openvswitch/scripts/ovs-lib
%{_datadir}/openvswitch/scripts/ovs-save
%{_datadir}/openvswitch/scripts/ovs-vtep
%{_datadir}/openvswitch/scripts/ovs-ctl
%{_datadir}/openvswitch/scripts/ovs-kmod-ctl
%{_datadir}/openvswitch/scripts/ovs-systemd-reload
%config %{_datadir}/openvswitch/local-config.ovsschema
%config %{_datadir}/openvswitch/vswitch.ovsschema
%config %{_datadir}/openvswitch/vtep.ovsschema
%{_bindir}/ovs-appctl
%{_bindir}/ovs-dpctl
%{_bindir}/ovs-ofctl
%{_bindir}/ovs-vsctl
%{_bindir}/ovsdb-client
%{_bindir}/ovsdb-tool
%{_bindir}/ovs-pki
%{_bindir}/vtep-ctl
%{_libdir}/*.so.*
%{_sbindir}/ovs-vswitchd
%{_sbindir}/ovsdb-server
%{_mandir}/man1/ovsdb-client.1*
%{_mandir}/man1/ovsdb-server.1*
%{_mandir}/man1/ovsdb-tool.1*
%{_mandir}/man5/ovsdb.5*
%{_mandir}/man5/ovsdb.local-config.5*
%{_mandir}/man5/ovsdb-server.5.*
%{_mandir}/man5/ovs-vswitchd.conf.db.5*
%{_mandir}/man5/vtep.5*
%{_mandir}/man7/ovsdb-server.7*
%{_mandir}/man7/ovsdb.7*
%{_mandir}/man7/ovs-actions.7*
%{_mandir}/man7/ovs-fields.7*
%{_mandir}/man8/vtep-ctl.8*
%{_mandir}/man8/ovs-appctl.8*
%{_mandir}/man8/ovs-ctl.8*
%{_mandir}/man8/ovs-dpctl.8*
%{_mandir}/man8/ovs-kmod-ctl.8.*
%{_mandir}/man8/ovs-ofctl.8*
%{_mandir}/man8/ovs-pki.8*
%{_mandir}/man8/ovs-vsctl.8*
%{_mandir}/man8/ovs-vswitchd.8*
%{_mandir}/man8/ovs-parse-backtrace.8*
%{_udevrulesdir}/91-vfio.rules
%doc LICENSE NOTICE README.rst NEWS rhel/README.RHEL.rst
%ifarch %{dpdkarches}
%doc %{dpdkdir}/README.DPDK-PMDS
%attr(750,openvswitch,hugetlbfs) %verify(not owner group) /var/log/openvswitch
%else
%attr(750,openvswitch,openvswitch) %verify(not owner group) /var/log/openvswitch
%endif
/var/lib/openvswitch
%ghost %attr(755,root,root) %verify(not owner group) %{_rundir}/openvswitch
%{_datadir}/openvswitch/bugtool-plugins/
%{_datadir}/openvswitch/scripts/ovs-bugtool-*
%{_bindir}/ovs-dpctl-top
%{_sbindir}/ovs-bugtool
%{_mandir}/man8/ovs-dpctl-top.8*
%{_mandir}/man8/ovs-bugtool.8*
%if (0%{?rhel} && 0%{?rhel} <= 7) || (0%{?fedora} && 0%{?fedora} < 29)
%{_sysconfdir}/sysconfig/network-scripts/ifup-ovs
%{_sysconfdir}/sysconfig/network-scripts/ifdown-ovs
%endif

%if %{with ipsec}
%files ipsec
%{_datadir}/openvswitch/scripts/ovs-monitor-ipsec
%{_unitdir}/openvswitch-ipsec.service
%endif

%changelog
* Mon Jan 09 2023 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-27
- Merging upstream branch-3.0 [RH git: ef727f12fc]
    Commit list:
    6884a23186 cirrus: Update to use FreeBSD 12.4.


* Fri Jan 06 2023 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-26
- Merging upstream branch-3.0 [RH git: 38daf9f66e]
    Commit list:
    7b67053206 tc: Add support for TCA_STATS_PKT64. (#1776816)
    d792a8ae61 Documentation: Fix links in maintainers.rst.
    eba56e7e69 Documentation: Fix links in the DPDK guide on physical ports.
    eac5d21a3f treewide: Don't use non-portable '==' with test command.
    2ef8ae836a dpif: Fix tunnel key set for IPv6 tunnels with SLOW_ACTION.


* Tue Jan 03 2023 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-25
- Merging upstream branch-3.0 [RH git: 1a824c34d8]
    Commit list:
    88ec8b452d ci: Fix overriding OPTS provided from the yml.
    7b5c4240c1 python: Fix datapath flow decoders.
    57049862fc Prepare for 3.0.4.
    726c7b7975 Set release date for 3.0.3.
    953fa9c400 lldp: Fix bugs when parsing malformed AutoAttach.
    9821588278 dpif-netdev: Use unmasked key when adding datapath flows.
    7aeb7f1ce0 odp-execute: Add check for L4 header size.
    2410b95597 ovsdb-cs: Consider default conditions implicitly acked.
    a984a22e5a odp-execute: Fix ipv4 missing clearing of connection tracking fields.
    c272928c8b faq: Update some wording since kernel module is already removed.
    52b85e54a7 rculist: Use rculist_back_protected to access prev.
    192db0d038 Prepare for 3.0.3.
    ea6c986ff7 Set release date for 3.0.2.
    5898bba3d7 odp-util: Fix reporting unknown keys as keys with bad length.
    c47b8fd5ab ovs-dpctl-top: Fix ovs-dpctl-top via pipe.


* Tue Dec 06 2022 Timothy Redaelli <tredaelli@redhat.com> - 3.0.0-24
- net/i40e: fix jumbo frame Rx with X722 [RH git: dfd86c1969]
    [ upstream commit 719469f13b11dbdc921b74258f2d10bd1c5328d4 ]
    
    For NIC I40E_10G-10G_BASE_T_X722, when the port is configured with
    link speed, it cannot receive jumbo frame packets.
    
    Because it set maximum frame size failed when starts the port that
    the port link status is still down.
    
    This patch fix the error that starts the port will force set maximum
    frame size.
    
    Fixes: 2184f7cdeeaa ("net/i40e: fix max frame size config at port level")
    
    Signed-off-by: Jie Wang <jie1x.wang@intel.com>
    Tested-by: Dukai Yuan <dukaix.yuan@intel.com>


* Thu Nov 24 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-23
- Merging upstream branch-3.0 [RH git: 82c17b1a8f]
    Commit list:
    ebf52fd371 rculist: Fix iteration macros.
    998fc998fe vswitchd: Publish per iface received multicast packets.
    0d31e85734 learn: Fix parsing immediate value for a field match.
    eccfce96b5 datapath-windows: Check the condition to reset pseudo header checksum on Rx side


* Thu Nov 10 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-22
- Merging upstream branch-3.0 [RH git: c7214a4776]
    Commit list:
    c9c602b6f3 netdev-offload-dpdk: Enhance the support of tunnel pop action


* Sat Nov 05 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-21
- Merging upstream branch-3.0 [RH git: 4215c1b14e]
    Commit list:
    e3963987ac ci: Update meson requirement for DPDK.


* Fri Nov 04 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-20
- Merging upstream branch-3.0 [RH git: 958300b9cb]
    Commit list:
    4b6c748f87 ovsdb: transaction: Fix weak reference leak.
    d78eef0892 ovsdb: transaction: Refactor assess_weak_refs.
    e17707fcd9 ovs-tcpdump: Cleanup mirror port on SIGHUP/SIGTERM.
    6690ed80ca netdev-linux: Fix inability to apply QoS on ports with custom qdiscs. (#2138339)
    bfc572f582 tc: Fix misaligned writes while parsing pedit.
    89f3dd0c2b odp-util: Add missing separator in format_odp_conntrack_action().
    4cdf15787e vswitch.xml: Fix the name of rstp-path-cost option.
    02ab8d169f mac-learning: Fix learned fdb entries not age out issue.
    ef6da6a8fc ofproto-dpif-xlate: Update tunnel neighbor when receive gratuitous ARP.
    48f19bdfb2 bond: Fix crash while logging not yet enabled member.
    b414275134 netdev-dpdk: Fix tx_dropped counters value.


* Wed Nov 02 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-19
- Merging upstream branch-3.0 [RH git: e16662b44a]
    Commit list:
    c7d9ee7062 tc: On last action use drop action attribute instead of pipe


* Tue Nov 01 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-18
- Merging upstream branch-3.0 [RH git: 079b3d5170]
    Commit list:
    4a2fe9f1e3 netdev-linux: Allow meter to work in tc software datapath when tc-policy is specified


* Mon Oct 31 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-17
- Merging upstream branch-3.0 [RH git: 7cc8355b08]
    Commit list:
    d2ebceaff9 Revert "tc: Fix stats dump when using same meter table"


* Wed Oct 26 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-16
- Merging upstream branch-3.0 [RH git: 1526299063]
    Commit list:
    24c5414eca unaligned: Correct the stats of packet_count and byte_count on Windows.
    892346cd23 tests: Fix filtering of whole-second durations.
    9586642185 netdev-offload: Set 'miss_api_supported' to be under netdev.


* Tue Oct 18 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-15
- Merging upstream branch-3.0 [RH git: 7bfbe844a1]
    Commit list:
    64b79581be cmap: Add thread fence for slot update.
    ae188cf7bb ofproto-dpif-xlate: Do not use zero-weight buckets in select groups.
    7cbcf3e42c github: Update versions of action dependencies.


* Tue Oct 11 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-14
- Merging upstream branch-3.0 [RH git: 8b1b13d296]
    Commit list:
    c0eb7f652b ovsdb: Fix race for datum JSON string reference counter. (#2133431)
    3e2596f19b ovs-tcpdump: Fix bond port unable to capture jumbo frames.
    a8e316aee3 json: Fix deep copy of objects and arrays.


* Fri Oct 07 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-13
- Merging upstream branch-3.0 [RH git: 0963868b99]
    Commit list:
    c4bb0d8db3 Prepare for 3.0.2.
    228600969c Set release date for 3.0.1.


* Fri Oct 07 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-12
- Merging upstream branch-3.0 [RH git: 8df8ec7aed]
    Commit list:
    50feac16fb Add support for OpenSSL 3.0 functions.
    61df71366c dhparams: Fix .c file generation with OpenSSL >= 3.0.


* Fri Oct 07 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-11
- Merging upstream branch-3.0 [RH git: 56f04b833e]
    Commit list:
    446018dfd7 daemon-unix: Fix file descriptor leak when monitor restarts child.
    4cc16e7f94 vconn: Allow ECONNREFUSED in refuse connection test.


* Wed Oct 05 2022 Timothy Redaelli <tredaelli@redhat.com> - 3.0.0-10
- redhat: get the NVR from spec file directly [RH git: 03acc76bac]
    Fixes: 339efe77c4c7 ("pkgtool: keep %{?dist} before added bz string")


* Tue Oct 04 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-9
- Merging upstream branch-3.0 [RH git: 02669b1699]
    Commit list:
    92d587a99f dpdk: Use DPDK 21.11.2 release.


* Wed Sep 28 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-8
- Merging upstream branch-3.0 [RH git: 61e65a32d2]
    Commit list:
    d7e3bf2d86 m4: Test avx512 for x86 only.
    e379e42d47 ovsdb-idl: Preserve references for rows deleted in same IDL run as their insertion. (#2126450)


* Wed Sep 21 2022 Timothy Redaelli <tredaelli@redhat.com> - 3.0.0-7
- vhost: fix virtqueue use after free on NUMA reallocation [RH git: 325605f78b]
    [ upstream commit 0b2a2ca35037d6a5168f0832c11d9858b8ae946a ]
    
    translate_ring_addresses (via numa_realloc) may change a virtio device and
    virtio queue.
    The virtqueue object must be refreshed before accessing the lock.
    
    Fixes: 04c27cb673b9 ("vhost: fix unsafe vring addresses modifications")
    
    Signed-off-by: David Marchand <david.marchand@redhat.com>
    Reviewed-by: Maxime Coquelin <maxime.coquelin@redhat.com>


* Tue Sep 20 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-6
- Merging upstream branch-3.0 [RH git: 178660f414]
    Commit list:
    a841d55e6b datapath-windows: Correct Geneve IPV6 header checksum parameter


* Mon Sep 19 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-5
- Merging upstream branch-3.0 [RH git: bcf6d63a29]
    Commit list:
    6636fc9fff python: idl: Fix idl.Row.__str__ method.


* Fri Sep 16 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-4
- Merging upstream branch-3.0 [RH git: ea3923ee6a]
    Commit list:
    3bcb21a885 bond: Avoid deadlock while updating post recirculation rules.
    97cc130848 ofproto-dpif-upcall: Add debug commands to pause/resume revalidators.
    f555cec368 test-list: Fix false-positive build failure with GCC 12.


* Wed Sep 14 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-3
- Merging upstream branch-3.0 [RH git: 6abf0f157b]
    Commit list:
    fc5a49045e tests: Fix tests with GNU grep 3.8.


* Tue Sep 13 2022 Open vSwitch CI <ovs-ci@redhat.com> - 3.0.0-2
- Merging upstream branch-3.0 [RH git: 9ffbd299ac]
    Commit list:
    7876489969 tc: Fix stats dump when using same meter table
    dcbdadaa56 cirrus: Upgrade to FreeBSD 13.1 image.
    02bfa84df4 netdev-linux: Skip some internal kernel stats gathering.
    5737cdcc89 tests: Use _DAEMONIZE macro's to start tcpdump.
    bb949d5955 ofproto-dpif-xlate: Fix error messages for nonexistent ports/recirc_ids.
    ee2254be27 ofproto-dpif-xlate: Optimize datapath action set by removing last clone action.
    d5932c8d73 ofproto-dpif-xlate: Clear tunnel wc bits if original packet is non-tunnel.
    887b122515 raft: Fix unnecessary periodic compactions.
    dbbe4c55e5 netdev-offload-tc: Parse tunnel options only for geneve ports.
    6e8da6fbb6 netdev-offload-tc: Add missing handling of the tunnel source port.
    737d2cb5ae netdev-offload-tc: Fix ignoring unknown tunnel keys.
    b32bee91d7 netdev-offload-tc: Use masks instead of keys while parsing tunnel attributes.
    417600e681 netdev-offload-tc: Explicitly handle mask for the tunnel destination port.
    fafb386a3f netdev-offload-tc: Fix the mask for tunnel metadata length.
    1123e5485d Prepare for 3.0.1.


* Thu Sep 08 2022 Timothy Redaelli <tredaelli@redhat.com> - 3.0.0-1
- redhat: Imported Red Hat build files. [RH git: 6d9c1ccd08]


